//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Aggregate/InterferenceTwin.cpp
//! @brief     Implements class InterferenceTwin.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Aggregate/InterferenceTwin.h"
#include "Base/Util/Assert.h"
#include <cmath>

InterferenceTwin::InterferenceTwin(const R3& direction, double mean_distance, double std_dev)
    : IInterference(0)
    , m_direction(direction)
    , m_distance(mean_distance)
    , m_std_dev(std_dev)
{
    validateOrThrow();
}

InterferenceTwin* InterferenceTwin::clone() const
{
    auto* result = new InterferenceTwin(m_direction, m_distance, m_std_dev);
    result->setPositionVariance(m_position_var);
    return result;
}

R3 InterferenceTwin::direction() const
{
    return m_direction;
}

double InterferenceTwin::meanDistance() const
{
    return m_distance;
}

double InterferenceTwin::stdDev() const
{
    return m_std_dev;
}

double InterferenceTwin::iff_without_dw(const R3 q) const
{
    ASSERT(m_validated);
    double q_proj = q.dot(m_direction.unit());
    return 1.0
           + std::exp(-q_proj * q_proj * m_std_dev * m_std_dev / 2.0)
                 * std::cos(q_proj * m_distance);
}

std::string InterferenceTwin::validate() const
{
    std::vector<std::string> errs;
    requestGt0(errs, m_distance, "Mean");
    requestGt0(errs, m_std_dev, "StdDev");
    if (!errs.empty())
        return jointError(errs);
    m_validated = true;
    return "";
}
